# -*- coding: utf-8 -*-
import unittest

def cls_name(obj):
    return obj.__class__.__name__

class RequestTest(unittest.TestCase):

    def _make_environ(self, *args, **kwargs):
        """Create a blank request and return the environment that it creates.
        """
        from webob.request import Request
        return Request.blank(*args, **kwargs).environ

    def test_post(self):
        """Tests the Request's ability to filter out non-OAuth parmeters from
        a POST."""
        post_data = dict(
            oauth_nonce='hsu94j3884jdopsl',
            oauth_timestamp='1191242090',
            oauth_consumer_key='dpf43f3p2l4k3l03',
            oauth_signature_method='PLAINTEXT',
            oauth_version='1.0',
            oauth_signature='kd94hf93k423kf44&',
            oauth_callback='http://printer.example.com/request_token_ready',
            dumbo="Elephant")
        # Create the environment based on the post data.
        environ = self._make_environ('http://example.com/request_token',
            POST=post_data)

        # Make the environment an OAuth aware request.
        from wsgioauth.request import Request
        req = Request(environ)

        # Verify the request does not contain the non-OAuth parameter.
        from webob.multidict import MultiDict
        del post_data['dumbo']
        self.failUnlessEqual(req.oauth_params, MultiDict(post_data))

    def test_header(self):
        """Tests the Request's ability to identify OAuth parmeters in the
        HTTP Authorization header."""
        auth_header = 'OAuth realm="http://example.com/",oauth_consumer_key=\
"dpf43f3p2l4k3l03",oauth_token="nnch734d00sl2jdk",oauth_signature_method="HM\
AC-SHA1",oauth_signature="tR3%2BTy81lMeYAr%2FFid0kMTYa%2FWM%3D",oauth_timest\
amp="1191242096",oauth_nonce="kllo9940pd9333jh",oauth_version="1.0"'
        # Create the environment with the Authorization header.
        environ = dict(HTTP_AUTHORIZATION=auth_header)
        environ = self._make_environ('http://example.com/request_token',
            environ=environ)

        # Create an OAuth request from the previously built wsgi environment.
        from wsgioauth.request import Request
        req = Request(environ)

        from webob.multidict import MultiDict
        from oauth2 import Request
        # Unparse the header message. The 'OAuth ' is sliced off the first
        #   part of the string because it doesn't contain any variable
        #   information.
        params = Request._split_header(auth_header[6:]) # ugh!
        self.failUnlessEqual(req.oauth_params, MultiDict(params))

    def test_get(self):
        query_string = 'oauth_consumer_key=dpf43f3p2l4k3l03&oauth_signature_\
method=PLAINTEXT&oauth_signature=kd94hf93k423kf44%26&oauth_timestamp=1191242\
090&oauth_nonce=hsu94j3884jdopsl&oauth_version=1.0&oauth_callback=http%3A%2F\
%2Fprinter.example.com%2Frequest_token_ready'
        # Create the environment based on the query string.
        environ = self._make_environ('http://example.com/request_token?%s' % 
            query_string)

        # Create an OAuth request from the environment.
        from wsgioauth.request import Request
        req = Request(environ)

        from webob.multidict import MultiDict
        from oauth2 import Request
        params = Request._split_url_string(query_string) # ugh!
        self.failUnlessEqual(req.oauth_params, MultiDict(params))


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import urllib
from oauth2 import Consumer
from webob import Request, Response

from wsgioauth import calls
from wsgioauth.provider import Application, Storage
from wsgioauth.utils import CALLS
from wsgioauth.provider import Storage, Token

ROUTES = {
    u'getConsumers': calls.getConsumers,
    u'getRequestTokens': calls.getRequestTokens,
    u'getAccessTokens': calls.getAccessTokens,
    u'provisionConsumer': calls.provisionConsumer,
    u'provisionRequestToken': calls.provisionRequestToken,
    u'provisionAccessToken': calls.provisionAccessToken,
    u'deleteConsumer': calls.deleteConsumer,
    u'deleteRequestToken': calls.deleteRequestToken,
    u'deleteAccessToken': calls.deleteAccessToken,
    }

def getMockStorage():
    from wsgioauth.provider import OAUTH_CLASSES
    OAUTH_CLASSES['consumer'] = Consumer
    OAUTH_CLASSES['request_token'] = Token
    return Storage


def echo_app(environ, start_response):
    """Simple app that echos a POST request"""
    req = Request(environ)
    resp = Response(urllib.urlencode(req.params))
    return resp(environ, start_response)

def echo_app_factory(*global_conf, **local_conf):
    return echo_app

STORAGE = None

def app_factory(*global_conf, **local_conf):
    CALLS.update(ROUTES)
    global STORAGE
    if STORAGE is None:
        storage_cls = getMockStorage()
        STORAGE = storage_cls(local_conf)
    def storage_lookup(environ, conf):
        return STORAGE
    return Application(storage_lookup, **local_conf)

def filter_factory(app, *global_conf, **local_conf):
    """This function returns a wsgioauth.provider.Filter services factory."""
    from wsgioauth.mock import getMockStorage
    global STORAGE
    if STORAGE is None:
        storage_cls = getMockStorage()
        STORAGE = storage_cls(local_conf)
    def storage_lookup(environ, conf):
        return STORAGE
    from wsgioauth.provider import Middleware
    return Middleware(app, storage_lookup, **local_conf)




# -*- coding: utf-8 -*-
from oauth2 import escape, SignatureMethod_HMAC_SHA1, SignatureMethod_PLAINTEXT

from wsgioauth.utils import get_normalized_parameters

class HMAC_SHA1(SignatureMethod_HMAC_SHA1):

    def signing_base(self, request, consumer, token):
        sig = (
            escape(request.method),
            escape(request.path_url),
            escape(get_normalized_parameters(request)),
        )

        key = '%s&' % escape(consumer.secret)
        if token:
            key += escape(token.secret)
        raw = '&'.join(sig)
        return key, raw

PLAINTEXT = SignatureMethod_PLAINTEXT



import os, sys
sys.path.insert(0, os.path.dirname(os.path.dirname(__file__)))
import pkg_resources
# FIXME pumazi: this stuff is defined twice, once in setup.py and once here. ugh!
pkg_resources.require('WebOb')
pkg_resources.require('oauth2')
pkg_resources.require('httplib2') # dependency of oauth2



# -*- coding: utf-8 -*-
import oauth2 # XXX pumazi: factor this out
from webob.multidict import MultiDict, NestedMultiDict
from webob.request import Request as WebObRequest

__all__ = ['Request']

class Request(WebObRequest):
    """The OAuth version of the WebOb Request.
    Provides an easier way to obtain OAuth request parameters
    (e.g. oauth_token) from the WSGI environment."""

    def _checks_positive_for_oauth(self, params_var):
        """Simple check for the presence of OAuth parameters."""
        checks = [ p.find('oauth_') >= 0 for p in params_var ]
        return True in checks

    @property
    def str_oauth_header(self):
        extracted = {}
        # Check for OAuth in the Header
        if 'authorization' in self.headers:
            auth_header = self.headers['authorization']
            # Check that the authorization header is OAuth.
            if auth_header[:6] == 'OAuth ':
                auth_header = auth_header.lstrip('OAuth ')
                try:
                    # Extract the parameters from the header.
                    extracted = oauth2.Request._split_header(auth_header)
                except:
                    raise Error('Unable to parse OAuth parameters from '
                        'the Authorization header.')
        return extracted

    @property
    def str_oauth_POST(self):
        extracted = {}
        if self._checks_positive_for_oauth(self.str_POST):
            extracted = dict([ (k, v,) for k, v in self.str_POST.iteritems()
                if (k.find('oauth_') >= 0) ])
        return extracted

    @property
    def str_oauth_GET(self):
        extracted = {}
        if self._checks_positive_for_oauth(self.str_GET):
            extracted = dict([ (k, v,) for k, v in self.str_GET.iteritems()
                if (k.find('oauth_') >= 0) ])
        return extracted

    def params(self):
        params = WebObRequest.params.fget(self)
        return NestedMultiDict(params, self.str_oauth_header)

    params = property(params, doc=WebObRequest.params.__doc__)

    @property
    def oauth_params(self):
        """Simple way to get the OAuth parameters without sifting through
        the entire stack of parameters.
        We check the header first, because it is low hanging fruit.
        However, it would be more efficient to check for the POSTed
        parameters, because the specification defines the POST method as the
        recommended request type before using GET or the Authorization
        header."""
        extracted = {}
        # OAuth in the Header
        extracted.update(self.str_oauth_header)
        # OAuth in a GET or POST method
        extracted.update(self.str_oauth_GET)
        extracted.update(self.str_oauth_POST)

        # Return the extracted oauth variables 
        return MultiDict(extracted)

    @property
    def nonoauth_params(self):
        """Simple way to get the non-OAuth parameters from the request."""
        oauth_param_keys = self.oauth_params.keys()
        return dict([(k, v) for k, v in self.params.iteritems() if k not in oauth_param_keys])



# -*- coding: utf-8 -*-
import unittest
import urllib
import xmlrpclib
import urlparse

from webob import Request, Response

class ProviderTestBase(unittest.TestCase):

    def _make_storage(self, config={}):
        from wsgioauth.mock import getMockStorage
        storage_cls = getMockStorage()
        storage = storage_cls(config)
        return storage

    def _make_data(self, cls, assignable):
        """Creates dummy token *like* object data from a given class. This
        method has been created to work with the Consumer and Token class."""
        # Create dummy data from the given class.
        dummy = {'key1': cls('key1', 'secret'),
                 'key2': cls('key2', 'secret'),
                 'key3': cls('key3', 'secret'),
                 }
        # Assign the dummy data to the given assignable
        assignable.update(dummy)
        return assignable

    def _make_request(self, path='/'):
        """Creates a blank request."""
        request = Request.blank(path)
        return request


class StorageTest(ProviderTestBase):
    """Tests the BaseStorage class."""

    def setUp(self):
        # Create a dummy storage
        self.storage = self._make_storage()

    def test_consumers(self):
        storage = self.storage
        # populate the storage with a few consumers
        from oauth2 import Consumer
        dummy_consumers = self._make_data(Consumer, storage._consumers)
        # consumers should look and feel like a dict
        self.failUnlessEqual(len(storage.consumers), len(dummy_consumers))
        for key in storage.consumers.keys():
            self.assert_(key in dummy_consumers)

        # add a consumer
        storage.consumers['key4'] = Consumer('key4', 'secret')
        self.assert_('key4' in storage.consumers)
        # add an invalid consumer
        self.failUnlessRaises(AssertionError, storage.set_consumers, 'key5', 'secret')

        # delete a consumer
        del storage.consumers['key1']
        self.assert_('key1' not in storage.consumers)

    def test_request_tokens(self):
        storage = self.storage
        # populate the storage with a few tokens
        from oauth2 import Token
        dummy_request_tokens = self._make_data(Token, storage._request_tokens)
        # request_tokens should look and feel like a dict
        self.failUnlessEqual(len(storage.request_tokens), len(dummy_request_tokens))
        for key in storage.request_tokens.keys():
            self.assert_(key in dummy_request_tokens)

        # add a token
        storage.request_tokens['key4'] = Token('key4', 'secret')
        self.assert_('key4' in storage.request_tokens)
        # add an invalid consumer
        self.failUnlessRaises(AssertionError, storage.set_request_tokens, 'key5', 'secret')

        # delete a token
        del storage.request_tokens['key1']
        self.assert_('key1' not in storage.request_tokens)

    def test_access_tokens(self):
        storage = self.storage
        # populate the storage with a few tokens
        from oauth2 import Token
        dummy_access_tokens = self._make_data(Token, storage._access_tokens)
        # access_tokens should look and feel like a dict
        self.failUnlessEqual(len(storage.access_tokens), len(dummy_access_tokens))
        for key in storage.access_tokens.keys():
            self.assert_(key in dummy_access_tokens)

        # add a token
        storage.access_tokens['key4'] = Token('key4', 'secret')
        self.assert_('key4' in storage.access_tokens)
        # add an invalid consumer
        self.failUnlessRaises(AssertionError, storage.set_access_tokens, 'key5', 'secret')

        # delete a token
        del storage.access_tokens['key1']
        self.assert_('key1' not in storage.access_tokens)

class ProviderApplicationCallsTest(ProviderTestBase):
    """Test the various XML-RPC calls that are the function behind the
    administrative application."""

    def setUp(self):
        # make a storage and populate it with a few consumers
        self.storage = self._make_storage()

        from oauth2 import Consumer, Token
        # populate the storage with a few consumers
        dummy_consumers = self._make_data(Consumer, self.storage._consumers)
        # ... and request tokens
        dummy_request_tokens = self._make_data(Token, self.storage._request_tokens)
        for key, token in dummy_request_tokens.iteritems():
            # setup the callback and verfier for each token
            token.set_callback('http://example.com/callback')
        # ... and access tokens
        dummy_access_tokens = self._make_data(Token, self.storage._access_tokens)
        for key, token in dummy_access_tokens.iteritems():
            pass

        # create the request
        self.request = self._make_request()
        self.request.headers['content-type'] = 'text/xml'

    def tearDown(self):
        del self.storage
        del self.request

    def _set_params(self, params):
        self.request.body = xmlrpclib.dumps(tuple([params]))

    def _call_and_extract(self, rpc_func):
        response = rpc_func(self.storage, self.request)
        return xmlrpclib.loads(response.body)[0][0]

    def test_getConsumers(self):
        """Retrieving the 'key2' consumer form the storage using
        the getConsumers call."""
        # add some details to the request
        self._set_params('key2')

        # trigger the call and test the response
        from wsgioauth.calls import getConsumers
        data = self._call_and_extract(getConsumers)
        consumer = self.storage.consumers['key2']
        self.failUnlessEqual(data['key2'], dict(key=consumer.key, secret=consumer.secret))

    def test_provisionConsumer(self):
        """Provisioning the 'key4' consumer using the provisionConsumer call,
        which also tests the storage's add_consumer method."""
        # add some details to the request
        consumer_data = {'key': 'key4', 'secret': 'secret'}
        self._set_params(consumer_data)

        # trigger the call and test the response
        from wsgioauth.calls import provisionConsumer
        data = self._call_and_extract(provisionConsumer)
        self.failUnlessEqual(data['key4'], consumer_data)

    def test_deleteConsumer(self):
        """Attempting to delete the 'key1' consumer from the storage using
        the deleteConsumer call."""
        # add some details to the request
        self._set_params('key1')

        # trigger the call and test the response
        from wsgioauth.calls import deleteConsumer
        data = self._call_and_extract(deleteConsumer)
        self.failUnless(data, "The deleteConsumer call should have returned True.")
        self.failUnless('key1' not in self.storage.consumers)

        # trigger the call again except this time without a real consumer key
        self._set_params('does not exist')
        self.failUnlessRaises(xmlrpclib.Fault, deleteConsumer, self.storage, self.request)

    def test_getRequestTokens(self):
        # add some details to the request
        self._set_params('key1')

        # trigger the call and test the response
        from wsgioauth.calls import getRequestTokens
        data = self._call_and_extract(getRequestTokens)
        request_token = self.storage.request_tokens['key1']
        self.failUnlessEqual(data['key1'],
            dict(key=request_token.key, secret=request_token.secret))

    def test_provisionRequestToken(self):
        """Provisioning the 'key4' request token using the
        provisionRequestToken call, which also tests the storage's
        add_request_token method."""
        # add some details to the request
        token_data = {'key': 'key4', 'secret': 'secret',
            'callback': 'http://example.com/ding_fries_are_done',
            'verifier': 'apple pie'}
        self._set_params(token_data)

        # trigger the call and test the response
        from wsgioauth.calls import provisionRequestToken
        data = self._call_and_extract(provisionRequestToken)
        self.failUnlessEqual(data, token_data)

    def test_deleteRequestToken(self):
        """Delete the 'key3' request token from the storage using
        the deleteRequestToken call."""
        # add some details to the request
        self._set_params('key3')

        # trigger the call and test the response
        from wsgioauth.calls import deleteRequestToken
        data = self._call_and_extract(deleteRequestToken)
        self.failUnless(data, "The deleteRequestToken call should have returned True.")
        self.failUnless('key3' not in self.storage.request_tokens)

        # trigger the call again except this time without a real consumer key
        self._set_params('does not exist')
        self.failUnlessRaises(xmlrpclib.Fault, deleteRequestToken, self.storage, self.request)

    def test_getAccessTokens(self):
        # add some details to the request
        self._set_params('key2')

        # trigger the call and test the response
        from wsgioauth.calls import getAccessTokens
        data = self._call_and_extract(getAccessTokens)
        access_token = self.storage.access_tokens['key2']
        self.failUnlessEqual(data['key2'],
            dict(key=access_token.key, secret=access_token.secret))

    def test_provisionAccessToken(self):
        """Provisioning the 'key10' access token using the
        provisionAccessToken call, which also tests the storage's
        add_access_token method."""
        # add some details to the request
        token_data = {'key': 'key10', 'secret': 'secret'}
        self._set_params(token_data)

        # trigger the call and test the response
        from wsgioauth.calls import provisionAccessToken
        data = self._call_and_extract(provisionAccessToken)
        self.failUnlessEqual(data, token_data)

    def test_deleteAccessToken(self):
        """Delete the 'key2' access token from the storage using
        the deleteAccessToken call."""
        # Add some details to the request
        self._set_params('key2')

        # Trigger the call and test the response.
        from wsgioauth.calls import deleteAccessToken
        data = self._call_and_extract(deleteAccessToken)
        self.failUnless(data, "The deleteRequestToken call should have "
            "returned True.")
        self.failUnless('key2' not in self.storage.access_tokens)

        # Trigger the call again, except this time without a real consumer
        #   key.
        self._set_params('does not exist')
        self.failUnlessRaises(xmlrpclib.Fault, deleteAccessToken,
            self.storage, self.request)


class MockHandler(object):
    def start_response(self, status, headers, exc_info=None):
        """Store these values for future examination."""
        self.status = status
        self.headers = headers
        self.exc_info = exc_info


class ProviderApplicationTest(ProviderTestBase):

    def _make_app(self):
        from wsgioauth.provider import Application
        storage = getattr(self, 'storage', None)
        if storage is None:
            storage = self._make_storage()
        def storage_factory(environ, config):
            return storage
        app = Application(storage_factory)
        return app

    def test_route_registration(self):
        """Test the registration of the various XML-RPC calls (routes)
        in the application. An HTTP 500 is considered a passing test,
        because it means that a route was found, but could not complete."""
        # Create a request factory for use in testing multiple paths.
        def create_request(call_name=''):
            """Creates a dummy request. This factory is only useful to this
            test."""
            req = self._make_request()
            req.content_type = 'text/xml'
            req.body = xmlrpclib.dumps(tuple(), call_name)
            return req
        app = self._make_app()
        # Register the calls with the global variables.
        from wsgioauth.utils import CALLS
        from wsgioauth.mock import ROUTES
        from wsgioauth import calls
        CALLS.update(ROUTES)
        # The handler is used as a dummy start_response catcher, it also
        #   helps grab some of the response that we are testing against
        handler = MockHandler()

        for route in ROUTES.keys():
            request = create_request(route)
            response_body = app(request.environ, handler.start_response)
            self.failIfEqual(handler.status, '404 Not Found',
                "Failed to route to the %s call." % route)


# SPEC 1.0a appendix A "Documentation and Registration"
# Consumer is http://printer.example.com
CONSUMER = "http://printer.example.com"
# Service Provider is http://photos.example.net (different
#   domain, notice the ".net")
SERVICE_PROVIDER = "https://photos.example.net"

# Declared by the Service Provider for use by the Consumer
CONSUMER_KEY = "dpf43f3p2l4k3l03"
CONSUMER_SECRET = "kd94hf93k423kf44"
# Defined by the Consumer in the token request
CALLBACK_URL = "http://printer.example.com/request_token_ready"

# Defined by the Service Provider
AUTHORIZATION = 'authorize'
AUTHORIZATION_URL = SERVICE_PROVIDER + "/" + AUTHORIZATION
# Defined by the Service Provider
REQUEST_TOKEN = 'request_token'
REQUEST_TOKEN_URL = SERVICE_PROVIDER + "/" + REQUEST_TOKEN
# Defined by the Service Provider
ACCESS_TOKEN = 'access_token'
ACCESS_TOKEN_URL = SERVICE_PROVIDER + "/" + ACCESS_TOKEN

# Requested parameters from the Consumer to the Service Provider
RESOURCE_PARAMETERS = {'file': 'vacation.jpg', 'size': 'original'}
# The Protected Resource is defined by the Service Provider
RESOURCE_URL = SERVICE_PROVIDER.replace('https', 'http') + "/photos"


class ProviderMiddlewareTestBase(ProviderTestBase):

    def _make_middleware(self):
        # Create the storage factory
        storage = getattr(self, 'storage', None)
        if storage is None:
            storage = self._make_storage()
        def storage_factory(environ, config):
            return storage

        # Make the filter (aka middleware)
        from wsgioauth.provider import Middleware
        from wsgioauth.mock import echo_app
        middleware = Middleware(echo_app, storage_factory)
        return middleware

    def _setup_signatures(self):
        # Setup signing methods
        from wsgioauth.signatures import HMAC_SHA1, PLAINTEXT
        self.signature_method_plaintext = PLAINTEXT()
        self.signature_method_hmac_sha1 = HMAC_SHA1()


# SPEC 1.0a section "Service Provider Issues an Unauthorized Request Token"
class ProviderMiddlewareTest(ProviderMiddlewareTestBase):

    def setUp(self):
        self.storage = self._make_storage()
        self.middleware = self._make_middleware()
        self._setup_signatures()
        # Setup the consumer
        self.consumer = self.storage.add_consumer(CONSUMER_KEY, CONSUMER_SECRET)

    def tearDown(self):
        del self.storage
        del self.middleware
        del self.consumer

    def _extract_address_and_query(self, url):
        import urlparse
        parts = urlparse.urlparse(url)
        scheme, netloc, path, params, query, fragment = parts[:6]
        address = scheme + '://' + netloc + path
        query_string = dict(urlparse.parse_qsl(query))
        return address, query_string

    def test_non_oauth_request(self):
        """Check for the proper erros when a non-oauth request is made."""
        data = dict(one=1, two=2)
        # Make the request for a request_token
        from wsgioauth.request import Request
        req = Request.blank(REQUEST_TOKEN_URL, POST=data)
        from wsgioauth.provider import NotAnOAuthRequest
        self.failUnlessRaises(NotAnOAuthRequest, self.middleware.process, self.storage, req)

    def test_partial_oauth_request(self):
        """Checks to see if the request will raise an error when only some of
        the OAuth parameters are sent."""
        data = dict(oauth_nonce='123456789',count='dracula')
        # Create the request for a request token
        from wsgioauth.request import Request
        req = Request.blank(REQUEST_TOKEN_URL, POST=data)
        from wsgioauth.provider import PartialOAuthRequest
        self.failUnlessRaises(PartialOAuthRequest, self.middleware.process, self.storage, req)

    def test_via_post(self):
        """Test the whole OAuth process using the HTTP POST method.
        1. Request Token
        2. Authorization
        3. Access Token
        4. Protected Resource
        
        There is no reason to check the storage for the token creation,
        because the later step will fail from the current steps inability to
        successfully store the token.
        """
        from wsgioauth.provider import Token
        from wsgioauth.request import Request
        # ===================================================================
        # 1. Request Token
        # ===================================================================

        # Setup the request params for an OAuth request token.
        # This is initialized by the consumer for a user new to the consumer,
        #   that has a previously expired access token or for a user that has
        #   previously declined the use of the access token.
        from oauth2 import generate_nonce, generate_timestamp
        data = {'oauth_callback': CALLBACK_URL,
                'oauth_consumer_key': self.consumer.key,
                'oauth_nonce': generate_nonce(),
                'oauth_signature': self.signature_method_plaintext.sign(
                    None, self.consumer, None),
                'oauth_signature_method': self.signature_method_plaintext.name,
                'oauth_timestamp': generate_timestamp(),
                'oauth_version': '1.0a',
                }

        # Create the request made by the consuemr for a request token.
        req = Request.blank(REQUEST_TOKEN_URL, POST=data)
        # Call the middlware with the consumer request.
        resp = self.middleware.process(self.storage, req)
        # Capture the request that has been given back to us.
        request_token = Token.from_string(resp.body)
        resp_data = urlparse.parse_qs(resp.body)
        # Check the response for a callback confirmation. This is done so
        #   that we know where the user will be sent after authorization.
        self.failUnlessEqual(request_token.callback_confirmed, 'true')

        # ===================================================================
        # 2. Authorization
        # ===================================================================

        # The consumer redirects the user's browser to the service provider's
        #   authorization URL.  The user will authenticate and allow or
        #   decline the consumer access to the protected resource.
        # We do not use authentication in this tests. Instead we use the
        #   no_auth plugin, which simply redirects the user to the callback
        #   URL.

        # Setup request params for an OAuth authorization, done by the user.
        # We send the service provider the OAuth token key so that it can
        #   identify the user with the token.
        data = {'oauth_token': request_token.key}

        # Create the request made by the user as directed by the consumer to
        #   the authorization URL.
        req = Request.blank(AUTHORIZATION_URL, POST=data)
        # Call the middlware with the user request, where the user would
        #   authenticate with the service provider.  See the notes above
        #   about the no_auth plugin for information about why we aren't
        #   sending any credentials.
        resp = self.middleware.process(self.storage, req)

        # Check for redirect to the consumer's callback URL.
        address, query_string = self._extract_address_and_query(resp.headers['location'])
        self.failUnlessEqual(address, CALLBACK_URL)

        # Verify the OAuth token is present.
        self.failUnlessEqual(query_string['oauth_token'], request_token.key)
        verifier = query_string['oauth_verifier']

        # ===================================================================
        # 3. Access Token
        # ===================================================================

        # The consumer can request an access token from the service provider
        #   after recieving a verifier from the user and/or upon a request to
        #   the callback URL.

        # Setup request params for an OAuth access token.
        # The access token is a product of the request token and user
        #   authorization.  Together these two items are used to obtain an
        #   access token, which will be used to access the protected
        #   resource.
        data = {'oauth_token': request_token.key,
                'oauth_consumer_key': self.consumer.key,
                'oauth_nonce': generate_nonce(),
                'oauth_signature': self.signature_method_plaintext.sign(None, self.consumer, request_token),
                'oauth_signature_method': self.signature_method_plaintext.name,
                'oauth_timestamp': generate_timestamp(),
                'oauth_verifier': verifier,
                'oauth_version': '1.0a',
                } # -- no verifier in SPEC version 1.0

        # Create the request made by the consumer to obtain an access token.
        req = Request.blank(ACCESS_TOKEN_URL, POST=data)
        # Call the middlware with the consumer request.
        resp = self.middleware.process(self.storage, req)
        access_token = Token.from_string(resp.body)

        # ===================================================================
        # 4. Protected Resource
        # ===================================================================

        # The consumer can access the protected resource on behave of the
        #   user by providing the access token as part of the request.

        # Setup request params for accessing the protected resource.
        data = {'oauth_token': access_token.key,
                'oauth_consumer_key': self.consumer.key,
                'oauth_nonce': generate_nonce(),
                'oauth_signature_method':
                    self.signature_method_hmac_sha1.name,
                'oauth_timestamp': generate_timestamp(),
                'oauth_version': '1.0a',
                }
        # Attach the resource specific parameters.
        data.update(RESOURCE_PARAMETERS)

        # Create the request made by the consumer for the protected resource.
        req = Request.blank(RESOURCE_URL, POST=data)
        # Sign the request using the HMAC SHA1 signature method.
        oauth_signature = self.signature_method_hmac_sha1.sign(req,
            self.consumer, access_token)
        # Update the parameters and recreate the request.
        data.update(dict(oauth_signature=oauth_signature))
        # The request needs to be created again because webob requests have
        #   read-only parameters, unless one wishes to override the
        #   wsgi.input environment key with new post data. It is far easier
        #   to recreate the request.
        # We create the request and sign it because the signature method
        #   builds a base signature from the request method, url and
        #   posted parameters without the oauth signature parameters itself.
        req = Request.blank(RESOURCE_URL, POST=data)

        # Create a mock request handler because we need to go through the
        #   middleware to the application and back. To do this we need
        #   something that will look like a valid start_response function.
        # Note that the middleware is being called directly, rather than
        #   using the process method as seen in the previvious steps. This is
        #   because we need to touch the application, which the process
        #   method does not do, but the middleware's __call__ does.
        handler = MockHandler()
        # Call the middleware
        body = self.middleware(req.environ, handler.start_response)
        # Grab the response data and parse it.  We are using an echo
        #   application to verify the request has successfully passed
        #   through the middleware.
        resp_data = body[0]
        encoded_params = urllib.urlencode(RESOURCE_PARAMETERS)
        # Verify the response matches what we sent to the echo application.
        self.failUnlessEqual(resp_data, encoded_params)

    def test_nonce(self):
        # Setup request params for an OAuth request token
        from oauth2 import generate_timestamp
        data = {'oauth_callback': CALLBACK_URL,
                'oauth_consumer_key': self.consumer.key,
                'oauth_nonce': '1234567890',
                'oauth_signature': self.signature_method_plaintext.sign(None,
                    self.consumer, None),
                'oauth_signature_method': self.signature_method_plaintext.name,
                'oauth_timestamp': generate_timestamp(),
                'oauth_version': '1.0a',
                }

        # Create the request
        from wsgioauth.request import Request
        req = Request.blank(REQUEST_TOKEN_URL, POST=data)
        # Call the middleware and grab the response, which should be valid.
        resp = self.middleware.process(self.storage, req)
        # Make the same request again, which should raise a NOnceReplayed
        #   error as a result of using the same nonce value twice.
        from wsgioauth.provider import NOnceReplayed
        self.failUnlessRaises(NOnceReplayed, self.middleware.process,
            self.storage, req)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
from urllib import urlencode
from urlparse import parse_qsl

import httplib2
import oauth2
from webob import Request, Response
from webob.exc import HTTPFound, HTTPNotFound, HTTPUnauthorized
try:
    from wsgiref.simple_server import make_server
except ImportError:
    raise ImportError("""
wsgiref is required to run this example, but not required by the wsgioauth
package.  You must install wsgiref before you can run this file.  One way to
install it is through easy_install by using the following command in your
console:  easy_install wsgiref
""")

from wsgioauth.consumer import Client

# The following are the actual example use-case URLs. For the purposes of
#   this example, we have reassigned the domains to the associated
#   application.
# CALLBACK_URL = "http://printer.example.com/request_token_ready"
# REQUEST_TOKEN_URL = "https://photos.example.net/request_token"
# AUTHORIZATION_URL = "http://photos.example.net/authorize"
# ACCESS_TOKEN_URL = # "https://photos.example.net/access_token"
# PROTECTED_RESOURCE_URL = "http://photos.example.net/photo"

CALLBACK_URL = "http://localhost:8081/request_token_ready"
# localhost:8080 is available by running protected_resource.py
REQUEST_TOKEN_URL = "http://localhost:8080/request_token"
AUTHORIZATION_URL = "http://localhost:8080/authorize"
ACCESS_TOKEN_URL = "http://localhost:8080/access_token"
PROTECTED_RESOURCE_URL = "http://localhost:8080/photo"
RESOURCE_PARAMETERS = {'file': 'vacation.jpg', 'size': 'original'}

consumer = ('dpf43f3p2l4k3l03', # key
            'kd94hf93k423kf44', # secret
            )

template = """<html><body>%s</body></html>"""

class ConsumerApp(object):
    """Example that does a single user consumption of the protected resource.
    This example application will not handle two or more OAuth processes
    at a time."""

    consumer = oauth2.Consumer(*consumer)
    request_token = None
    access_token = None

    def fetch_request_token(self):
        """Fetch a request token from the service provider.  The request
        token is used to obtain authorization."""
        # Get a client and give it a callback.  If the call back isn't
        #   given it will be set to 'oob', which means out-of-band. But if
        #   the callback hasn't really been registered out-of-band, then an
        #   error will be raised.
        client = self.get_client(callback_url=CALLBACK_URL)

        resp, content = client.request(REQUEST_TOKEN_URL)
        # Capture the request token
        self.request_token = oauth2.Token.from_string(content)

    def fetch_access_token(self, verifier):
        # Assign the verifier to the request token so that the client can use
        #   it without explicitly being told to do so.
        self.request_token.verifier = verifier

        # Get a client that will use the consumer and request token to obtain
        #   the access token. 
        client = self.get_client()

        resp, content = client.request(ACCESS_TOKEN_URL)
        # Capture the access token
        self.access_token = oauth2.Token.from_string(content)
        # Null out the request token, because it is now useless anyways.
        self.request_token = None

    def get_client(self, **kwargs):
        """Create a client from the most recent information."""
        token = None

        # Determines which token we should use, if any.
        if self.access_token is not None:
            token = self.access_token
        elif self.request_token is not None:
            token = self.request_token

        return Client(self.consumer, token, **kwargs)

    def index_page(self, request):
        """The front-page or index page... click the link on this page to
        initialize the OAuth process.  Return to the page to see the saved
        access token."""
        # Create a link to the OAuth *thing*. In this case the resouce is
        #   similar to RPC, but could just as well be an actual file.
        body = """<a href="/print_vacation">Print my vacation image</a>"""
        body += "<br />"

        # Display any information we might have about the consumer and access
        #   token.  This is very unsecure of course, but handy for display
        #   purposes.
        body += ("<div><b>consumer</b>: <i>key</i> = %s, <i>secret</i> = "
            "%s</div>" % (self.consumer.key, self.consumer.secret))
        if self.access_token is not None:
            body += ("<div><b>access token</b>: <i>key</i> = %s, "
                "<i>secret</i> = %s</div>" % (self.access_token.key,
                self.access_token.secret))

        body = template % body
        return Response(body)

    def print_vacation(self, request):
        """Uses OAuth to do... well nothing sophisticated, but hopefully
        you will get the point.  If the access token doesn't exist, we
        initialize the process for obtaining it."""
        if self.access_token is None:
            # The access token is required to gain access to the protected
            #   resource.
            if self.request_token is None:
                self.fetch_request_token()
                # Redirect the user to the service provider's authorization
                #   URL with the token key as payload; so that the service
                #   provider can associate the user with a particular request
                #   token.
                location = '?'.join([AUTHORIZATION_URL, "oauth_token=%s" %
                    self.request_token.key])
                response = HTTPFound(location=location)
            else:
                # okay... why are you here? O.o
                response = HTTPFound("%s/" % request.host_url)
        else:
            # Because we have an access token, we can now make a request to
            #   the protected resource. 
            client = self.get_client()

            resp, content = client.request(PROTECTED_RESOURCE_URL, "POST",
                urlencode(RESOURCE_PARAMETERS))

            # Create the response body for this page based on the protected
            #   resource data.
            body = ""
            resp_data = dict(parse_qsl(content))
            # Display the protected resource's response data on the page.
            body += '<br />'.join([ "<div><b>%s</b></div>%s</div>" %
                (k, v) for k, v in resp_data.iteritems() ])

            # Create a link back to the index page.
            body += """<br /><a href="/">Back to the index page.</a>"""

            body = template % body
            response = Response(body)

        return response

    def __call__(self, environ, start_response):
        # Create a WebOb like request object.
        request = Request(environ)

        # Route the path to the associated method.
        if request.path_info == '/':
            response = self.index_page(request)
        elif request.path_info == '/print_vacation':
            response = self.print_vacation(request)
        elif request.path_info == '/request_token_ready':
            # The request_token is ready. This simply means that we have all
            #   the information we need to use the request token to retrieve
            #   the access token.
            self.fetch_access_token(request.params['oauth_verifier'])
            response = HTTPFound(location='%s/print_vacation' %
                request.host_url)
        else:
            response = HTTPNotFound()
        return response(environ, start_response)


if __name__ == '__main__':
    app = ConsumerApp()
    server = make_server('localhost', 8081, app)
    print "Serving the oauth consumer application at http://localhost:8081/"
    server.serve_forever()



# -*- coding: utf-8 -*-
# Lifted from repoze.bfg.xmlrpc!
import xmlrpclib
import webob

__all__ = ['xmlrpc_marshal', 'xmlrpc_response', 'parse_xmlrpc_request',
    'xmlrpc',]

def xmlrpc_marshal(data):
    """ Marshal a Python data structure into an XML document suitable
    for use as an XML-RPC response and return the document.  If
    ``data`` is an ``xmlrpclib.Fault`` instance, it will be marshalled
    into a suitable XML-RPC fault response."""
    if isinstance(data, xmlrpclib.Fault):
        return xmlrpclib.dumps(data)
    else:
        return xmlrpclib.dumps((data,),  methodresponse=True)

def xmlrpc_response(data):
    """ Marshal a Python data structure into a webob ``Response``
    object with a body that is an XML document suitable for use as an
    XML-RPC response with a content-type of ``text/xml`` and return
    the response."""
    xml = xmlrpc_marshal(data)
    response = webob.Response(xml)
    response.content_type = 'text/xml'
    response.content_length = len(xml)
    return response

def parse_xmlrpc_request(request):
    """ Deserialize the body of a request from an XML-RPC request
    document into a set of params and return a two-tuple.  The first
    element in the tuple is the method params as a sequence, the
    second element in the tuple is the method name."""
    if request.content_length > (1 << 23):
        # protect from DOS (> 8MB body)
        raise ValueError('Body too large (%s bytes)' % request.content_length)
    params, method = xmlrpclib.loads(request.body)
    return params, method

def xmlrpc(wrapped):
    """ Function meant to be used as a decorator.  The ``xmlrpc``
    function turns functions which accept params and return Python
    structures into functions.  The decorated function must accept a
    ``context`` argument and zero or more positional arguments
    (conventionally named ``*params``).

    E.g.::

      from wsgioauth.xmlrpc import xmlrpc

      @xmlrpc
      def say(context, what):
          if what == 'hello'
              return {'say':'Hello!'}
          else:
              return {'say':'Goodbye!'}

    Equates to::

      from wsgioauth.xmlrpc import parse_xmlrpc_request
      from wsgioauth.xmlrpc import xmlrpc_response

      def say_view(context, request):
          params, method = parse_xmlrpc_request(request)
          return say(context, *params)

      def say(context, what):
          if what == 'hello'
              return {'say':'Hello!'}
          else:
              return {'say':'Goodbye!'}

    Note that if you use ``repoze.bfg.convention``, you must decorate your
    view function in the following order for it to be recognized by the
    convention machinery as a view::

      @xmlrpc
      def say(context, what):
          if what == 'hello'
              return {'say':'Hello!'}
          else:
              return {'say':'Goodbye!'}

    """
    
    def _curried(context, request):
        params, method = parse_xmlrpc_request(request)
        value = wrapped(context, *params)
        return xmlrpc_response(value)
    _curried.__name__ = wrapped.__name__
    _curried.__grok_module__ = wrapped.__module__ # r.bfg.convention support

    return _curried


# -*- coding: utf-8 -*-
import urlparse
try:
    from urlparse import parse_qs, parse_qsl
except ImportError:
    from cgi import parse_qs, parse_qsl

import httplib2
import oauth2
from oauth2 import Consumer, Token

__all__ = ['Request', 'Client']

class Request(oauth2.Request):
    """OAuth version 1.0a request class."""
    version = '1.0a'

    @classmethod
    def from_consumer_and_token(cls, consumer, token=None,
        http_method=oauth2.HTTP_METHOD, http_url=None,
        parameters=None, callback=None):
        if not parameters:
            parameters = {}

        defaults = {
            'oauth_consumer_key': consumer.key,
            'oauth_timestamp': cls.make_timestamp(),
            'oauth_nonce': cls.make_nonce(),
            'oauth_version': cls.version,
        }

        defaults.update(parameters)
        parameters = defaults

        if token:
            parameters['oauth_token'] = token.key
            if token.verifier:
                # we have, so use it then null it
                parameters['oauth_verifier'] = token.verifier
                token.verifier = None
        else: # must be a request_token
            if callback is None and callback != 'oob':
                if not hasattr(consumer, 'callback'):
                    raise RuntimeError('Callback url is undefined.')
                else:
                    callback = 'oob'
            parameters['oauth_callback'] = callback

        return Request(http_method, http_url, parameters)


class Client(httplib2.Http):
    """OAuth version 1.0a client."""

    callback_url = 'oob' # out-of-band

    def __init__(self, consumer, token=None, cache=None, timeout=None,
        proxy_info=None, callback_url=None):

        if consumer is not None and not isinstance(consumer, Consumer):
            raise ValueError("Invalid consumer.")

        if token is not None and not isinstance(token, Token):
            raise ValueError("Invalid token.")

        self.consumer = consumer
        self.token = token
        self._signature = oauth2.SignatureMethod_HMAC_SHA1()

        if callback_url is not None:
            self.callback_url = callback_url

        httplib2.Http.__init__(self, cache=cache, timeout=timeout,
            proxy_info=proxy_info)

    def get_signature(self):
        return self._signature

    def set_signature(self, signature):
        if not isinstance(signature, SignatureMethod):
            raise ValueError("Invalid signature method.")
        self._signature = signature

    signature = property(get_signature, set_signature)

    def request(self, uri, method="GET", body=None, headers=None,
        redirections=httplib2.DEFAULT_MAX_REDIRECTS, connection_type=None):

        if not isinstance(headers, dict):
            headers = {}

        if body and method == "POST":
            parameters = dict(parse_qsl(body))
        elif method == "GET":
            parsed = urlparse.urlparse(uri)
            parameters = parse_qs(parsed.query)
        else:
            parameters = None

        req = Request.from_consumer_and_token(self.consumer, token=self.token,
            http_method=method, http_url=uri, parameters=parameters,
            callback=self.callback_url)

        req.sign_request(self.signature, self.consumer, self.token)

        if method == "POST":
            body = req.to_postdata() 
            headers['Content-Type'] = 'application/x-www-form-urlencoded'
        elif method == "GET":
            uri = req.to_url()
        else:
            headers.update(req.to_header())

        return httplib2.Http.request(self, uri, method=method, body=body,
            headers=headers, redirections=redirections,
            connection_type=connection_type)



# -*- coding: utf-8 -*-
import urlparse
import xmlrpclib
from time import time

import pkg_resources
from oauth2 import (
    generate_verifier,
    Consumer as OAuthConsumer,
    Error,
    MissingSignature,
    Token as OAuthToken,
    )
from webob import Request, Response
from webob.exc import (
    HTTPBadRequest,
    HTTPFound,
    HTTPNotFound,
    HTTPServerError,
    HTTPTemporaryRedirect,
    HTTPUnauthorized,
    )
from wsgioauth.request import Request as OAuthRequest
from wsgioauth.utils import (
    generate_string,
    repackage_request,
    xmlrpc_router,
    NOnceList,
    )

__all__ = ['Application', 'Filter', 'Consumer', 'Token',
    'OAUTH_CLASSES', 'VERIFIER',]

# TODO pumazi: OAUTH_CLASSES and VERIFIER needs defined from egg-entry-points
OAUTH_CLASSES = {'consumer': OAuthConsumer}

Consumer = OAuthConsumer

class Token(OAuthToken):
    """Simple subclass of the original oauth2.Token for the purpose
    of overriding the constructor and the verifier setter."""

    def __init__(self, key, secret, callback=None, verifier=None, **kwargs):
        self.key = key
        self.secret = secret
        self.verifier_generator = kwargs.get('verifier_generator', generate_string)

        if self.key is None or self.secret is None:
            raise ValueError("Key and secret must be set.")
        if callback is not None:
            self.set_callback(callback)
        if verifier is not None:
            self.set_verifier(verifier)

    def set_verifier(self, verifier=None):
        if verifier is not None:
            self.verifier = verifier
        else:
            self.verifier = self.verifier_generator()

    def get_callback_url(self):
        if self.callback and self.verifier:
            # Append the oauth_verifier.
            parts = urlparse.urlparse(self.callback)
            scheme, netloc, path, params, query, fragment = parts[:6]
            if query:
                query = "%s&oauth_token=%s&oauth_verifier=%s" % (query, self.key, self.verifier)
            else:
                query = "oauth_token=%s&oauth_verifier=%s" % (self.key, self.verifier)
            return urlparse.urlunparse((scheme, netloc, path, params, query, fragment))
        return self.callback


OAUTH_CLASSES.update({'request_token': Token, 'access_token': Token})

class BaseStorage(object):
    """The storage abstraction."""

    def __init__(self, config):
        # parse the config vars based on your storage implementation
        self.config = config

    consumers = {} # property()
    request_tokens = {} # property()
    access_tokens = {} # property()

    def _key_secret_generator(self, container_name, key, secret):
        if key is None:
            while True:
                key = generate_string()
                if key not in getattr(self, container_name): break
        if secret is None:
            secret = generate_string(128)
        return key, secret

    def add_consumer(self, key=None, secret=None, **kwargs):
        """Abstraction for adding a consumer without knowing the Consumer class."""
        key, secret = self._key_secret_generator('consumers', key, secret)
        consumer_cls = OAUTH_CLASSES['consumer']
        return self._create_consumer(consumer_cls, key, secret, **kwargs)

    def _create_consumer(self, cls, key, secret, **kwargs):
        """Storage dependent implmentation here."""
        raise NotImplementedError

    def add_request_token(self, key=None, secret=None, **kwargs):
        """Abstraction for adding a consumer without knowing the Consumer class."""
        key, secret = self._key_secret_generator('request_tokens', key, secret)
        request_token_cls = OAUTH_CLASSES['request_token']
        return self._create_request_token(request_token_cls, key, secret, **kwargs)

    def _create_request_token(self, cls, key, secret, **kwargs):
        """Storage dependent implmentation here."""
        raise NotImplementedError

    def add_access_token(self, key=None, secret=None, **kwargs):
        """Abstraction for adding a consumer without knowing the Consumer class."""
        key, secret = self._key_secret_generator('access_tokens', key, secret)
        access_token_cls = OAUTH_CLASSES['access_token']
        return self._create_access_token(access_token_cls, key, secret, **kwargs)

    def _create_access_token(self, cls, key, secret, **kwargs):
        """Storage dependent implmentation here."""
        raise NotImplementedError


class Storage(BaseStorage):
    """A mock storage abstraction."""

    _consumers = None
    _request_tokens = None
    _access_tokens = None

    def __init__(self, config):
        # parse the config vars based on your storage implementation
        self.config = config
        self._consumers = {}
        self._request_tokens = {}
        self._access_tokens = {}

    # consumers property
    def get_consumers(self, key=None):
        if key is not None:
            return self._consumers[key]
        else:
            return self._consumers
    def set_consumers(self, key, value):
        # check that the value is a Consumer object
        assert isinstance(value, Consumer), "The value is a an oauth2.Consumer instance."
        self._consumers[key] = value
    def del_consumers(self, key=None):
        if key is not None:
            del self._consumers[key]
        else:
            self._consumers = {}
    consumers = property(get_consumers, set_consumers, del_consumers,
        "OAuth consumer dictionary")

    def _create_consumer(self, cls, key, secret, **kwargs):
        """Storage dependent implmentation here."""
        consumer = cls(key, secret, **kwargs)
        self._consumers[key] = consumer
        return self.consumers[key]

    # request_tokens property
    def get_request_tokens(self, key=None):
        if key is not None:
            return self._request_tokens[key]
        else:
            return self._request_tokens
    def set_request_tokens(self, key, value):
        # check that the value is a Consumer object
        assert isinstance(value, Token), "The value is a an oauth2.Token instance."
        self._request_tokens[key] = value
    def del_request_tokens(self, key=None):
        if key is not None:
            del self._request_tokens[key]
        else:
            self._request_tokens = {}
    request_tokens = property(get_request_tokens, set_request_tokens, del_request_tokens,
        "OAuth request tokens dictionary")

    def _create_request_token(self, cls, key, secret, **kwargs):
        """Storage dependent implmentation here."""
        request_token = cls(key, secret, **kwargs)
        self._request_tokens[key] = request_token
        return self.request_tokens[key]

    # access_tokens property
    def get_access_tokens(self, key=None):
        if key is not None:
            return self._access_tokens[key]
        else:
            return self._access_tokens
    def set_access_tokens(self, key, value):
        # check that the value is a Consumer object
        assert isinstance(value, Token), "The value is a an oauth2.Token instance."
        self._access_tokens[key] = value
    def del_access_tokens(self, key=None):
        if key is not None:
            del self._access_tokens[key]
        else:
            self._access_tokens = {}
    access_tokens = property(get_access_tokens, set_access_tokens, del_access_tokens,
        "OAuth request tokens dictionary")

    def _create_access_token(self, cls, key, secret, **kwargs):
        """Storage dependent implmentation here."""
        access_token = cls(key, secret, **kwargs)
        self._access_tokens[key] = access_token
        return self.access_tokens[key]


class Application(object):
    """The application is used to setup and administer consumers and tokens."""

    def __init__(self, storage_factory, **config):
        self.storage_factory = storage_factory
        self.config = config

    def __call__(self, environ, start_response):
        """The typical WSGI application callable."""
        response = self.publish(environ)
        return response(environ, start_response)

    def publish(self, environ):
        request = Request(environ)
        # xml-rpc only!
        if request.content_type.find('text/xml') >= 0:
            # FIXME pumazi: aww... this sucks! using the repoze.bfg.xmlrpc
            #   (xmlrpc module) code was a mistake :(
            #   The result of this is unloading the request twice =/
            params, call_name = xmlrpclib.loads(request.body)
            storage = self.storage_factory(environ, self.config)
            # route the call
            call = xmlrpc_router(call_name)
            if call is not None:
                try:
                    response = call(storage, request)
                except Exception, e:
                    response = HTTPServerError(str(e))
            else:
                response = HTTPNotFound()
        else:
            response = HTTPNotFound()

        return response


def no_auth(request, token):
    """No authentication, just pass along"""
    callback = token.get_callback_url()
    return HTTPTemporaryRedirect(location=callback), None

def user_grabber(request, token):
    """grab the user from the request"""
    user = request.environ.get('HTTP_X_REMOTE_USER', None)
    return None, user


class NotAnOAuthRequest(Error):
    """Happens when a request has been found not to be OAuth like."""


class PartialOAuthRequest(Error):
    """There are some oauth parameters present in the request, but one or
    more is are missing."""


class UnknownSignature(Error):
    """Error indicating that an unknown signing method was used on the
    request."""


class NOnceReplayed(Error):
    """Error signals that the n-once value has already been used within a
    certain threshold."""


class InvalidVerifier(Error):
    """Error signallying that the verifier provided is invalid, because it
    either does not match or was not provided."""


class Middleware(object):
    """The middleware is used to authorize users attempting to
    access the protected resource (the application). This middleware attempts
    to adhere to the OAuth 1.0a specification."""

    signature_methods = {}

    def __init__(self, app, storage_factory, **config):
        self.app = app
        self.storage_factory = storage_factory
        self._config = config


        # setup the authentication plugin
        auth_plugin_name = 'no_auth' # default!
        if 'auth_plugin_name' in self._config:
            auth_plugin_name = self._config['auth_plugin_name']

        auth_plugin = None
        for p in pkg_resources.iter_entry_points('wsgioauth_authentication', auth_plugin_name):
            auth_plugin = p
            break
        if auth_plugin is None:
            raise ValueError("The provided authentication plugin %s could not be found." % auth_plugin_name)
        self.auth_plugin = auth_plugin.load()

        # setup the signature methods
        for plugin in pkg_resources.iter_entry_points('wsgioauth_signatures'):
            signature_cls = plugin.load()
            self.signature_methods[signature_cls.name] = signature_cls()

        # FIXME: the nonce list heavily depends on traffic usage
        nonce_cache_size = self._config.get('nonce_cache_size', 20000)
        self.nonce_list = NOnceList(nonce_cache_size)

    def __call__(self, environ, start_response):
        # 1. get the webob request and oauth request
        req = OAuthRequest(environ)
        try:
            response = self.process(self.get_storage(environ), req)
        except Error, e:
            response = HTTPBadRequest(e.message)
        except:
            response = HTTPServerError("An unknown problem has occured, "
                "please contact your site administrator for help with this "
                "issue.")
        return response(environ, start_response)

    def get_storage(self, environ):
        """Transparent access to the storage without worrying about
        the factory."""
        return self.storage_factory(environ, self._config)

    def process(self, storage, request):
        """Process the OAuth request."""
        # 2. determine if the request is an oauth request
        consumer_key = request.oauth_params.get('oauth_consumer_key', None)
        oauth_token_key = request.oauth_params.get('oauth_token', None)
        if len(request.oauth_params) == 0: # not an oauth reqeust
            raise NotAnOAuthRequest("The request made does not contain one "
                "or more OAuth parameters.")
        elif not (consumer_key or oauth_token_key):
            # an oauth request requires one of the above parameters?
            raise PartialOAuthRequest("A consumer or token was not provided"
                "in the request.")

        # 3. determine what is being asked for
        context = request.path.split('/')[-1]
        oauth_routes = {
            'request_token': self.process_request_token,
            'authorize': self.process_authorize,
            'access_token': self.process_access_token,
            }
        if context in oauth_routes:
            response = oauth_routes[context](storage, request)
        else:
            # request for a protected resource
            response = self.process_resource(storage, request)

        # 5. either start the response along or respond to it now
        return response

    def _check_nonce(self, nonce):
        if nonce in self.nonce_list:
            raise NOnceReplayed("The provided nonce value has been used recently.")
        self.nonce_list.append(nonce)

    def _check_timestamp(self, timestamp, threshold=300):
        """Check the timestamp is within the threshold."""
        if timestamp is None:
            raise Error("The oauth_timestamp parameter is missing.")
        timestamp = int(timestamp)
        now = int(time())
        lapsed = now - timestamp
        if lapsed > threshold:
            raise Error('Expired timestamp: given %d and now %s has a '
                'greater difference than threshold %d' % (timestamp, now, threshold))

    def _check_signature(self, request, consumer, token):
        """Determines if the request was made with good signing practices"""
        try:
            nonce = request.oauth_params['oauth_nonce']
        except KeyError:
            raise PartialOAuthRequest("Missing oauth_nonce.")
        self._check_nonce(nonce)

        # was the request made within a predefined window of time
        try:
            timestamp = request.oauth_params['oauth_timestamp']
        except KeyError:
            raise PartialOAuthRequest("Missing oauth_timestamp.")
        self._check_timestamp(timestamp)

        # TODO: scheme to signature verification (e.g. http to HMAC-SHA1)
        # get the signature method
        signature_name = request.oauth_params.get('oauth_signature_method')
        # get the signing method from the dictionary of known signing methods
        try:
            signature_method = self.signature_methods[signature_name]
        except KeyError:
            raise UnknownSignature("%s is a signature method unknown to this"
                " application." % signature_method)

        # get the signature from the request
        try:
            signature = request.oauth_params['oauth_signature']
        except KeyError:
            raise MissingSignature('The oauth_signature is missing.')

        # validate the signature
        valid = signature_method.check(request, consumer, token, signature)
        if not valid:
            key, base = signature_method.signing_base(request, consumer, token)
            raise Error('Invalid signature. Expected signature base string: %s' % base)

    def process_request_token(self, storage, request):
        """Process a request from the consumer for a request token."""
        # get a request token
        consumer = storage.consumers[request.oauth_params['oauth_consumer_key']]
        self._check_signature(request, consumer, None)

        token = storage.add_request_token()

        # verify the callback
        callback = request.oauth_params.get('oauth_callback', None)
        if callback:
            if callback == 'oob': # out-of-band
                if hasattr(consumer, 'callback'):
                    token.set_callback(consumer.callback)
                else:
                    raise PartialOAuthRequest("There is no callback set for "
                        "out-of-band (oob) use.")
            else:
                token.set_callback(callback)
        else:
            # let's be nice and see if there is an out-of-band callback
            if hasattr(consumer, 'callback'):
                token.set_callback(consumer.callback)
            else:
                raise PartialOAuthRequest("Missing oauth_callback. Required "
                    "by OAuth 1.0a.")

        # send an okay response
        return Response(body=token.to_string(), content_type="text/plain")

    def process_authorize(self, storage, request):
        """Process a request by the user for authorization and verification."""
        token = storage.request_tokens[request.oauth_params['oauth_token']]
        token.set_verifier()
        resp, user = self.auth_plugin(request.environ, token)
        # TODO: set the user to the token?
        return resp

    def process_access_token(self, storage, request):
        """Process a request by the consumer for an access token."""
        # get the access token
        consumer = storage.consumers[request.oauth_params['oauth_consumer_key']]
        request_token = storage.request_tokens[request.oauth_params['oauth_token']]
        try:
            verifier = request.oauth_params['oauth_verifier']
        except KeyError:
            raise InvalidVerifier("Missing oauth_verifier. Required by "
                "OAuth 1.0a.")

        self._check_signature(request, consumer, request_token)

        # Verify the token verifier, which is given to the user during
        #   authorization.  The verifier is then requested by the consumer to
        #   be sent in this request.
        if verifier != request_token.verifier:
            raise InvalidVerifier("Invalid verifier.")

        access_token = storage.add_access_token()
        # send an okay response
        return Response(body=access_token.to_string(), content_type="text/plain")

    def process_resource(self, storage, request):
        """Process a request for a resource."""
        # verify the request has been oauth authorized
        consumer = storage.consumers[request.oauth_params['oauth_consumer_key']]
        token = storage.access_tokens[request.oauth_params['oauth_token']]
        self._check_signature(request, consumer, token)
        # repackage the request without the oauth params before sending it to the application
        new_request = repackage_request(request)
        # send an okay response
        response = new_request.get_response(self.app)
        # TODO: set the Authorization header; should make the request less confusing?
        return response



# -*- coding: utf-8 -*-
import unittest
from urllib import urlencode

class SignatureMethodComparisonTest(unittest.TestCase):

    def test_signature_base_for_hmac_sha1(self):
        """Tests that that the custom version of the HMAC SHA1 signature
        method produces the same results as the original."""
        from oauth2 import Consumer
        consumer = Consumer('dpf43f3p2l4k3l03', 'kd94hf93k423kf44')

        # Define the request information
        method = "POST"
        url = "https://photos.example.net/request_token"
        params = dict(
            oauth_nonce='hsu94j3884jdopsl',
            oauth_timestamp='1191242090',
            oauth_consumer_key=consumer.key,
            oauth_signature_method='PLAINTEXT',
            oauth_version='1.0a',
            oauth_callback='http://printer.example.com/request_token_ready',
            dumbo="Elephant")

        # Create the requests, one from oauth2 and one from this package.
        from oauth2 import Request as OAuthRequest
        oauth_request = OAuthRequest(method, url, params)
        from wsgioauth.request import Request
        webob_request = Request.blank(url, POST=params)

        # Quickly verify the contents of the requests are in the right place.
        self.failUnlessEqual(webob_request.method, method)
        self.failUnlessEqual(webob_request.path_url, url)
        self.failUnlessEqual(oauth_request.method, method)
        self.failUnlessEqual(oauth_request.url, url)

        # Setup the signatures.
        from oauth2 import SignatureMethod_HMAC_SHA1
        oauth_sig_method = SignatureMethod_HMAC_SHA1()
        from wsgioauth.signatures import HMAC_SHA1
        wsgioauth_sig_method = HMAC_SHA1()

        # Sign the requests.
        oauth_request_sig_base = oauth_sig_method.signing_base(
            oauth_request, consumer, None)
        webob_request_sig_base = wsgioauth_sig_method.signing_base(
            webob_request, consumer, None)
        self.failUnlessEqual(oauth_request_sig_base, webob_request_sig_base)

        # Define the request information again except this time using the GET
        #   method.
        method = "GET"

        # Create the requests.
        oauth_request = OAuthRequest(method, url, params)
        webob_request = Request.blank('?'.join([url, urlencode(params)]))

        # Sign the requests.
        oauth_request_sig_base = oauth_sig_method.signing_base(
            oauth_request, consumer, None)
        webob_request_sig_base = wsgioauth_sig_method.signing_base(
            webob_request, consumer, None)
        self.failUnlessEqual(oauth_request_sig_base, webob_request_sig_base)

        # Define the request information again except this time using the
        #   HTTP Authorization header.
        # Create the requests.
        del params['dumbo']
        auth_header = 'OAuth realm=http://example.com,' + ','.join([ '%s=%s' % (k,v) for k,v in params.iteritems() ])
        webob_request = Request.blank('?'.join([url, 'dumbo=Elephant']),
            headers={'authorization': auth_header})

        # Sign the requests.
        webob_request_sig_base = wsgioauth_sig_method.signing_base(
            webob_request, consumer, None)
        self.failUnlessEqual(oauth_request_sig_base, webob_request_sig_base)




# -*- coding: utf-8 -*-
import xmlrpclib
from oauth2 import Consumer, Token
from wsgioauth.xmlrpc import xmlrpc

__all__ = (
    'getConsumers', 'getRequestTokens', 'getAccessTokens',
    'provisionConsumer', 'provisionRequestToken', 'provisionAccessToken',
    'deleteConsumer', 'deleteRequestToken', 'deleteAccessToken',
    )

def get_tokens(container, keys=[]):
    tokens = {}
    if keys == []:
        keys = container.keys()
    for k, v in container.iteritems():
        if k in keys:
            tokens[k] = dict(key=v.key, secret=v.secret)
    return tokens

# 
# The following are get type XML-RPC functions.
# 

@xmlrpc
def getConsumers(store, keys=[]):
    """A way to list all the consumers."""
    return get_tokens(store.consumers, keys)

@xmlrpc
def getRequestTokens(store, keys=[]):
    """List all or grab one or more request token in the storage."""
    return get_tokens(store.request_tokens, keys)

@xmlrpc
def getAccessTokens(store, keys=[]):
    """List all or grab one or more access token in the storage."""
    return get_tokens(store.access_tokens, keys)

# 
# The following XML-RPC functions are used to provision consumers and tokens.
# 

@xmlrpc
def provisionConsumer(store, params={}):
    """Provision a consumer and return the key and secret."""
    # retrieve the variables from the params argument
    key = params.get('key', None)
    secret = params.get('secret', None)
    # title = params.get('title', '')
    # description = params.get('description', '')
    # email = params.get('email', None)

    # call the storage abstration to add a consumer
    consumer = store.add_consumer(key, secret) #, title=title, description=description)
    return {consumer.key: {'key': consumer.key, 'secret': consumer.secret}}

@xmlrpc
def provisionRequestToken(store, params={}):
    """provision a token and return the key and secret"""
    # retrieve the variables from the params argument
    # key = params.get('key', None)
    # secret = params.get('secret', None)
    # callback = params.get('callback', None)
    # verifier = params.get('verifier', None)

    token = store.add_request_token(**params)
    response_data = {'key': token.key, 'secret': token.secret}
    if token.callback:
        response_data['callback'] = token.callback
    if token.verifier:
        response_data['verifier'] = token.verifier

    return response_data

@xmlrpc
def provisionAccessToken(store, params={}):
    """provision a token and return the key and secret"""
    # retrieve the variables from the params argument
    # key = params.get('key', None)
    # secret = params.get('secret', None)
    # callback = params.get('callback', None)
    # verifier = params.get('verifier', None)

    token = store.add_access_token(**params)
    response_data = {'key': token.key, 'secret': token.secret}
    return response_data

# 
# The following are used to delete consumers and tokens.
# 

@xmlrpc
def deleteConsumer(store, key):
    """Delete a consumer from the storage."""
    try:
        del store.consumers[key]
    except KeyError:
        raise xmlrpclib.Fault(-1, "The requested consumer does not exist.")
    # according to the xml-rpc spec, we need to return something
    return True

@xmlrpc
def deleteRequestToken(store, key):
    """Delete a request token from the storage."""
    try:
        del store.request_tokens[key]
    except KeyError:
        raise xmlrpclib.Fault(-1, "The requested request token does not exist.")
    # according to the xml-rpc spec, we need to return something
    return True

@xmlrpc
def deleteAccessToken(store, key):
    """Delete an access token from the storage."""
    try:
        del store.access_tokens[key]
    except KeyError:
        raise xmlrpclib.Fault(-1, "The requested access token does not exist.")
    # according to the xml-rpc spec, we need to return something
    return True



# -*- coding: utf-8 -*-
try:
    import wsgiref
except ImportError:
    raise ImportError("""
wsgiref is required to run this example, but not required by the wsgioauth
package.  You must install wsgiref before you can run this file.  One way to
install it is through easy_install by using the following command in your
console:  easy_install wsgiref
""")
from wsgiref.simple_server import make_server
from wsgioauth.mock import app_factory, echo_app, filter_factory

consumer = ('dpf43f3p2l4k3l03', # key
            'kd94hf93k423kf44', # secret
            )

if __name__ == '__main__':
    # Create the administration app.  The administration app isn't used in
    #   this example, but you may want to try using it outside of the
    #   example.
    admin_app = app_factory()

    # Instantiate the storage so that we can add the example consumer.
    mock_storage = admin_app.storage_factory({}, {})
    mock_storage.add_consumer(*consumer)

    # Create the OAuth protected application.
    oauth_protected_app = filter_factory(echo_app)

    def mapper(environ, start_response):
        """Routes the request to the correct application."""
        if environ['PATH_INFO'].find('oauth_admin') >= 0:
            return admin_app(environ, start_response)
        else:
            return oauth_protected_app(environ, start_response)
    server = make_server('localhost', 8080, mapper)
    print("Serving the echo application at:  http://localhost:8080/")
    print("Serving the admin application at:  "
        "http://localhost:8080/oauth_admin")
    server.serve_forever()



import os

from setuptools import setup, find_packages

here = os.path.abspath(os.path.dirname(__file__))
README = open(os.path.join(here, 'README.txt')).read()
CHANGES = open(os.path.join(here, 'CHANGES.txt')).read()
LICENSE = open(os.path.join(here, 'LICENSE.txt')).read()

__version__ = '0.3'
__name__ = 'wsgioauth'
__description__ = """
A WSGI OAuth for working with OAuth from the consumer and service provider
perspective. Contains a middleware for protected WSGI applications with an
associated administration application. Provides a simple library to help
consumers integrate OAuth protected resources into their applications."""
__author__ = 'Michael Mulich | WebLion Group, Penn State University'
__email__ = 'support@weblion.psu.edu'

install_requires = [
    'webob',
    'oauth2',
    'httplib2', # dependency of oauth2
    ]
tests_require = []

setup(
    name=__name__,
    version=__version__,
    description=__description__,
    long_description='\n\n'.join([README, CHANGES, LICENSE]),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Programming Language :: Python",
        "Topic :: Security",
        "Topic :: Internet :: WWW/HTTP",
        "Topic :: Internet :: WWW/HTTP :: WSGI",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Application",
        "Topic :: Internet :: WWW/HTTP :: WSGI :: Middleware",
        ],
    author=__author__,
    author_email=__email__,
    url='http://weblion.psu.edu/',
    license='GPL', # http://www.gnu.org/licenses/gpl.txt
    keywords='web wsgi oauth authentication authorization',
    packages=find_packages(),
    namespace_packages=[__name__],
    include_package_data=True,
    zip_safe=False,
    install_requires=install_requires,
    tests_require=tests_require,
    test_suite="tests",
    entry_points = """\
    [paste.app_factory]
    mock_oauth_admin = %(name)s.mock:app_factory
    mock_echo_app = %(name)s.mock:echo_app_factory
    [paste.filter_app_factory]
    mock_oauth_filter = %(name)s.mock:filter_factory

    [wsgioauth_authentication]
    no_auth = %(name)s.provider:no_auth
    user_grabber = %(name)s.provider:user_grabber
    [wsgioauth_signatures]
    plaintext = %(name)s.signatures:PLAINTEXT
    hmac_sha1 = %(name)s.signatures:HMAC_SHA1
    """ % {'name': __name__,}
    )



# -*- coding: utf-8 -*-
import os, random, string
import urllib
from collections import deque
from cStringIO import StringIO

from oauth2 import Error

__all__ = ['generate_string', 'random_word', 'xmlrpc_router',
    'get_normalized_parameters', 'repackage_request', 'NOnceList']

def generate_string(n=32):
    """Generate random hash *like* strings"""
    return ''.join([ random.choice(string.hexdigits) for x in xrange(n) ])

def random_word():
    """flipped out of:
    http://mail.python.org/pipermail/tutor/2001-June/006301.html
    WARNING to Windows users... bah, not worth the time :P
    """
    stat = os.stat('/usr/share/dict/words')
    # the filesize if the 7th element of the array
    flen = stat[6]
    f = open('/usr/share/dict/words')
    word = None
    while word is None:
        # seek to a random offset in the file
        f.seek(int(random.random() * flen))
        # do a single read with sufficient characters
        chars = f.read(50)
        # split it on white space
        wrds = string.split(chars)
        # the first element may be only a partial word so use the second
        # you can also make other tests on the word here
        if len(wrds[1]) > 5 and len(wrds[1]) < 9:
           word = wrds[1]
    return word

CALLS = {}

def xmlrpc_router(call_name):
    if call_name in CALLS.keys():
        return CALLS[call_name]
    return None

def get_normalized_parameters(request):
    """Return a string that contains the parameters that must be signed."""
    items = [ (k, v) for k, v in request.params.iteritems() if k != 'oauth_signature' ]
    encoded_str = urllib.urlencode(sorted(items))
    # Encode signature parameters per Oauth Core 1.0 protocol
    # spec draft 7, section 3.6
    # (http://tools.ietf.org/html/draft-hammer-oauth-07#section-3.6)
    # Spaces must be encoded with "%20" instead of "+"
    return encoded_str.replace('+', '%20')

def repackage_request(request):
    """Repackages the request into a request without oauth parameters."""
    new_request = request.copy()
    # first we need to deterime where the oauth stuff is in the request;
    #   then remove it without corrupting other request data

    # check for oauth in the Header
    if 'authorization' in request.headers:
        # check that the authorization header is oauth
        if request.headers['authorization'][:6] == 'OAuth ':
            del new_request.headers['authorization']

    # doesn't hurt to check in both places
    params = [(k,v) for k,v in request.params.iteritems() if not k.startswith('oauth_')]
    if request.method == 'GET': # modify the query string
        new_request.environ['QUERY_STRING'] = urllib.urlencode(params)
    elif request.method == 'POST': # modify the wsgi.input
        new_request.environ['wsgi.input'] = StringIO(urllib.urlencode(params))

    return new_request


class NOnceList(deque):
    """A ring buffer of nonce values"""

    def __init__(self, max_size=20000):
        deque.__init__(self)
        self.max_size = max_size

    def __setitem__(self, k, value):
        length = len(self)
        if length >= self.max_size or k > self.max_size:
            self.popleft()
            self.append(value)
        else:
            deque.__setitem__(self, k, value)

    def append(self, v):
        deque.append(self, v)
        if len(self) > self.max_size:
            self.popleft()
